/* contact-source.vala
 *
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;
using Gee;
using PeopleBackend;

namespace PeopleBackendDummy {

	public class ContactSource : PeopleBackendHelper.AbstractContactSource,
			PeopleBackend.DumpableContactSource {

		[Notify]
		public ContactSourceFlags flags {
			get;
			protected set;
			default = ContactSourceFlags.CAN_CREATE_CONTACT;
		}

		private Gee.Set<Contact> contacts;

		construct {
			contacts = new HashSet<Contact>();
			contacts.add (new Contact("foo-bar", contacts));
			contacts.add (new Contact("my-uncle", contacts));
		}

		public ContactSource() {
		}

		public override AsyncOperation<PeopleBackend.Contact> request_contact(string? id = null) {
			Contact contact = null;

			if (id == null)
				contact = new Contact (null, contacts);
			else {
				foreach (Contact c in contacts) {
					if(c.id == id) {
						contact = c;
						break;
					}
				}

				if(contact == null)
					return sync_error(new ContactSourceError.INVALID_ARGUMENT("No such contact : " + id));
			}

			bind_contact(contact);
			return sync_result(contact);
		}

		/* DumpableContactSource interface */
		public PeopleBackend.AsyncSearch<Contact> dump() {
			var search = new AsyncSearch<Contact>();

			foreach(Contact contact in contacts)
				search.add_hit(contact);
			search.finish();

			return search;
		}
		/* end of DumpableContactSource interface */

	}
}
