/* contact.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;
using Gee;
using PeopleBackend;
using PeopleBackendHelper;

namespace PeopleBackendDummy {

	public class Contact : PeopleBackendHelper.AbstractContact {
		private Gee.Map<string, string> fields;

		private ContactFieldSpec default_field_spec =
				new ContactFieldSpec(ContactFieldFlags.WRITABLE);

		private ContactFieldSpec mandatory_field_spec =
				new ContactFieldSpec(ContactFieldFlags.WRITABLE | ContactFieldFlags.MANDATORY);

		public weak Gee.Set<Contact> contacts {
			get;
			construct;
		}

		construct {
			flags = ContactFlags.EDITABLE | ContactFlags.DELETABLE;
			fields = new Gee.HashMap<string, string> (str_hash, str_equal, str_equal);
		}

		public Contact(string? id, Gee.Set<Contact> contacts) {
			this.id = id;
			this.contacts = contacts;
		}

		public override AsyncOperation<Gee.Set<string> > get_defined_fields() {
			return sync_result(fields.get_keys());
		}

		public override AsyncOperation<Gee.Set<string> > get_definable_fields() {
			return sync_result(new Gee.HashSet<string>(str_hash, str_equal));
		}

		public override AsyncOperation<Gee.Set<string> > get_mandatory_fields() {
			var result = new Gee.HashSet<string>(str_hash, str_equal);
			result.add ("identity.name.first");
			result.add ("identity.name.last");
			return sync_result(result);
		}

		public override AsyncOperation<ContactFieldSpec> get_field_spec(string field_name) {
			if (field_name == "identity.name.first" || field_name == "identity.name.last")
				return sync_result(mandatory_field_spec);
			return sync_result(default_field_spec);
		}

		public override AsyncOperation<Gee.Map<string, string> >
				get_fields(Gee.Set<string> field_names) {
			var result = new HashMap<string, string> (str_hash, str_equal, str_equal);
			foreach (weak string name in field_names) {
				if (fields.contains(name))
					result.set(name, fields.get(name));
			}
			return sync_result(result);
		}

		public override AsyncOperation<Gee.Set<string> >
				set_fields(Gee.Map<string, string> fields) {
			if (id == null) {
				if (!fields.contains ("identity.name.first") || !fields.contains ("identity.name.last"))
					return sync_error(new ContactError.MISSING_MANDATORY_FIELD("Missing mandatory parameters"));
				id = fields.get("identity.name.first") + "-" + fields.get("identity.name.last");
				contacts.add(this);
			}

			foreach (weak string name in fields.get_keys()) {
				this.fields.set(name, fields.get(name));
			}
			var op = sync_result(fields.get_keys());
			return op;
		}

		public override AsyncOperation<bool> @delete() {
			var op = new AsyncOperation<bool>();
			if (id == null) {
				op.completed_with_result (false);
			} else {
				contacts.remove(this);
				fields.clear();
				id = null;
				op.completed_with_result (true);
			}
			return op;
		}

	}

}
