using GLib;
using Gee;
using PeopleBackend;

namespace PeopleBackendSqliteTest {
	class QueryableContactSource : TestIt.Suite {
		private string db_path = "/tmp/people-test-queryable-contact-source.db";
		private Sqlite.Database db;

		private PeopleBackend.ContactSource contact_source;
		private PeopleBackend.Query query;

		private uint contacts_target_count;
		private uint contacts_current_count;

		protected override void initialize () {
			name = "PeopleBackendSqlite::QueryableContactSource";
			add_test_case ("query", test_query);
		}

		protected override void set_up () {
			Sqlite.Database.open(db_path, out db);
			db.exec (
				"CREATE TABLE IF NOT EXISTS people(" +
					"id integer PRIMARY KEY AUTOINCREMENT," +
					"first_name string," +
					"middle_name string," +
					"last_name string," +
					"home_phone string," +
					"work_phone string," +
					"mobile_phone string," +
					"home_email string," +
					"work_email string" +
				");");
			db.exec ("INSERT INTO people (first_name, last_name) VALUES (\"Foo\", \"Bar\")");
			db.exec ("INSERT INTO people (first_name, last_name) VALUES (\"Foo\", \"Foo\")");
			db.exec ("INSERT INTO people (first_name, last_name) VALUES (\"Faa\", \"Bar\")");
			db.exec ("INSERT INTO people (first_name, last_name) VALUES (\"Foo\", \"Bur\")");

			var factory = new PeopleBackendSqlite.ContactSourceFactory ();
			var parameters = new Gee.HashMap<string, string> (str_hash, str_equal, str_equal);
			parameters.set ("db-path", "/tmp/people-test-queryable-contact-source.db");
			contact_source = factory.request_contact_source (parameters);

			var q0 = new Gee.ArrayList<Query> ();
			var q1 = new Gee.ArrayList<Query> ();
			var q2 = new Gee.ArrayList<Query> ();

			q1.add (new FieldQuery("identity.name.first"));
			q1.add (new FieldQuery("identity.name.last"));
			q0.add (Query.not (new CompositeQuery (Operator.EQUALS, q1)));

			q2.add (new FieldQuery ("identity.name.first"));
			q2.add (new ValueQuery ("Foo"));
			q0.add (new CompositeQuery (Operator.CONTAINS, q2));

			query = Query.and (q0);

			contacts_current_count = 0;
			contacts_target_count = 2;
		}

		protected override void tear_down () {
			contact_source = null;

			db = null;
			FileUtils.unlink (db_path);
		}

		private void test_query () {
			var search = ((PeopleBackend.QueryableContactSource) contact_source).execute (query);
			search.hit_found += (src, contact) => {
						contacts_current_count++;
					};
			search.done += src => {
						assert_true (contacts_current_count == contacts_target_count);
						contacts_current_count = 0;
					};
			search.start();
		}

		static int main(string[] args) {
			var suite = new QueryableContactSource ();
			return suite.run();
		}
	}
}
