/* contact.vala
 *
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;
using Gee;


namespace PeopleService {

	[DBus (name = "net.launchpad.People.Contact")]
	public class Contact : PeopleService.DBusObject {

		// FIXME: we want to export this as a DBus property when dbus-glib is fixed
		[DBus (hidden = true)]
		public string id {
			get;
			private set construct;
		}

		[DBus (hidden = true)]
		public People.Contact contact {
			get;
			private set construct;
		}

		[DBus (hidden = true)]
		public weak AddressBook address_book {
			get;
			private set construct;
		}

		public signal void defined_fields_retrieved (string fields);
		public signal void fields_retrieved (string fields);

		public Contact (DBus.Connection connection,
				string id, People.Contact contact,
				AddressBook address_book) {
			this.connection = connection;
			this.object_path = "/net/launchpad/People/Contact/%s/%s".printf (address_book.id, clean_id (id));

			this.id = id;
			this.contact = contact;
			this.address_book = address_book;
		}

		public string GetId () throws Error {
			return id;
		}

		public void Close () throws Error {
			// FIXME: unref this contact from the contact manager so that it gets
			// removed from the bus when the reference drops to 0
		}

		public void RequestDefinedFields () throws Error {
			PeopleBackend.AsyncOperationCallback cb = (op, data) => {
				Gee.Set<string> fields = (Gee.Set<string>) op.get_result ();
				/*string[] result = new string[fields.size];

				uint i = 0;
				foreach (string field in fields) {
					result[i] = field;
					i++;
				}*/

				StringBuilder result = new StringBuilder ();
				foreach (string field in fields) {
					result.append (field + " ");
				}

				defined_fields_retrieved (result.str);
			};

			contact.get_defined_fields ().add_callback (cb);
		}

		public void RequestFields (string fields) throws Error {
			PeopleBackend.AsyncOperationCallback cb = (op, data) => {
				Gee.Map<string, string> fields = (Gee.Map<string, string>) op.get_result ();
				if (false) {
				string[] result = new string[fields.size];

				uint i = 0;
				foreach (string field_name in fields.get_keys ()) {
					result[i] = "%s: %s".printf (field_name, fields.get (field_name));
					i++;
				}
				}
				StringBuilder result = new StringBuilder ();
				foreach (string field_name in fields.get_keys ()) {
					result.append ("%s: %s\n".printf (field_name, fields.get (field_name)));
				}

				fields_retrieved (result.str);
			};

			Gee.Set<string> fields_set = new Gee.HashSet<string> (str_hash, str_equal);
			foreach (string field in fields.split(" ")) {
				fields_set.add (field);
			}

			contact.get_fields (fields_set).add_callback (cb);
		}

		private static string clean_id (string id) {
			var regex = new Regex ("[^a-zA-Z0-9_]");
			return regex.replace_literal (id, -1, 0, "_");
		}
	}

}
