/* query-result.vala
 *
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;
using Gee;


namespace PeopleService {

	[DBus (name = "net.launchpad.People.QueryResult")]
	public class QueryResult : PeopleService.DBusObject {

		// FIXME: we want to export this as a DBus property when dbus-glib is fixed
		[DBus (hidden = true)]
		public string id {
			get;
			private set construct;
		}

		// FIXME: we want to export this as a DBus property when dbus-glib is fixed
		[DBus (hidden = true)]
		public uint pending_results {
			get {
				return query_result.pending_results;
			}
		}

		[DBus (hidden = true)]
		public People.QueryResult query_result {
			get;
			private set construct;
		}

		[DBus (hidden = true)]
		public weak AddressBook address_book {
			get;
			private set construct;
		}

		public signal void found_results (uint results_count);
		public signal void done ();

		construct {
			query_result.ready += query_result_ready;
			query_result.finished += query_result_finished;
		}

		public QueryResult (DBus.Connection connection,
				string id, People.QueryResult query_result,
				AddressBook address_book) {
			this.connection = connection;
			this.object_path = "/net/launchpad/People/QueryResult/%s/%s".printf (address_book.id, id);

			this.id = id;
			this.query_result = query_result;
			this.address_book = address_book;
		}

		public void Open () throws Error {
			query_result.open ();
		}

		public void Close () throws Error {
			query_result.close ();
			address_book.query_results.remove (this);
		}

		public uint GetPendingResultsCount () throws Error {
			return pending_results;
		}

		[NoArrayLength]
		public void GetPendingResults (uint count, out string[] results) throws Error {
			var contacts = query_result.issue (count);
			var contact_manager = address_book.contact_manager;

			if (contacts.size > 0) {
				results = new string[contacts.size];
				uint i = 0;
				foreach (People.Contact contact in contacts) {
					var c = contact_manager.get_service_contact (contact);
					results[i] = c.object_path;
					i++;
				}
			}
		}

		private void query_result_ready (People.QueryResult result, uint results_count) {
			found_results (results_count);
		}

		private void query_result_finished (People.QueryResult result) {
			done ();
		}

	}

}
