/* self-test.vala
 *
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;

public class SelfTest : Object {
	public uint total {
		get;
		private set;
		default = 0;
	}

	public uint succeeded {
		get;
		private set;
		default = 0;
	}

	protected bool started = false;
	protected Timer timer = new Timer();

	public string name {
		get;
		construct;
	}

	public SelfTest (string name) {
		this.name = name;
	}

	public bool start () {
		assert (started == false);
		started = true;
		timer.start();
		stdout.printf (">> Test suite: %s\n", name);
		return true;
	}

	public bool end () {
		assert (started == true);
		timer.stop();
		started = false;
		stdout.printf ("  >>> Summary: %s\n", name);
		stdout.printf ("    %u/%u test(s) succeeded\n", succeeded, total);
		stdout.printf ("    test(s) executed in %f second(s)\n\n", timer.elapsed());
		return succeeded == total;
	}

	public void success (string title, string? message=null) {
		stdout.printf ("  > check #%02u:  %s OK", total++, ellipsize(title));
		if (message != null) {
			stdout.printf (" [%s]", message);
		}
		stdout.printf ("\n");
		succeeded++;
	}

	public void failure (string title, string? message=null) {
		stdout.printf ("  > check #%02u:  %s FAILED", total++, ellipsize(title));
		if (message != null) {
			stdout.printf (" [%s]", message);
		}
		stdout.printf ("\n");
	}

	public void assert_true (string title, bool cond, string? message=null) {
		if (cond) {
			success (title, message);
		} else {
			failure (title, message);
		}
	}

	public void assert_false (string title, bool cond, string? message=null) {
		if (! cond) {
			success (title, message);
		} else {
			failure (title, message);
		}
	}

	private static string ellipsize (string s) {
		string result = s.substring (0, 57);
		result +=  " " + string.nfill (61 - result.len(), '.');
		return result;
	}
}
