/* async-search.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;
using Gee;


namespace PeopleUtils {

	public class MultiAsyncSearch<S, T> : Object {
		private uint search_count;

		public Gee.Map<AsyncSearch<T>, S> searches {
			private get;
			private set construct;
		}

		public signal void hit_found (S source, T hit);
		public signal void done ();

		construct {
			assert (searches.size > 0);

			search_count = searches.size;

			foreach(AsyncSearch<T> search in searches.get_keys ()) {
				search.hit_found += hit_found_callback;
				search.done += done_callback;
			}
		}

		public MultiAsyncSearch(Gee.Map<AsyncSearch, S> searches) {
			this.searches = searches;
		}

		private void hit_found_callback (AsyncSearch<T> search, T hit) {
			hit_found (searches.get (search), hit);
		}

		private void done_callback (AsyncSearch<T> search) {
			if(--search_count == 0)
				done();
		}

		public void start () {
			foreach(AsyncSearch<T> search in searches.get_keys ())
				search.start ();
		}

		public void stop () {
			foreach(AsyncSearch<T> search in searches.get_keys ())
				search.stop ();
		}
	}

}
