using GLib;
using PeopleUtils;


namespace PeopleTest {

	private errordomain TestAsyncOperationError {
		RANDOM_ERROR = 1,
	}

	class AsyncOperation : TestIt.Suite {

		PeopleUtils.AsyncOperation<string> op;
		bool callback1_called;
		bool callback2_called;
		bool callback3_called;

		protected override void initialize () {
			name = "PeopleUtils::AsyncOperation";
			add_test_case ("early callback", test_early_callback);
			add_test_case ("late callback", test_late_callback);
			add_test_case ("middle callback", test_middle_callback);
			add_test_case ("error callback", test_error_callback);
		}

		protected override void set_up () {
			op = new PeopleUtils.AsyncOperation<string>();
			callback1_called = false;
			callback2_called = false;
			callback3_called = false;

			base.set_up ();
		}

		protected override void tear_down () {
			op = null;
			base.tear_down ();
		}

		private void test_early_callback () {
			op.completed_with_result ("result");
			op.add_callback (callback1);
			op.add_callback (callback2, "hello world");

			assert_true (callback1_called);
			assert_true (callback2_called);
		}

		private void test_middle_callback () {
			op.add_callback (callback1);
			op.completed_with_result ("result");

			assert_true (callback1_called);
			assert_false (callback2_called);

			op.add_callback (callback2, "hello world");

			assert_true (callback1_called);
			assert_true (callback2_called);
		}

		private void test_late_callback () {
			op.add_callback (callback1);
			op.add_callback (callback2, "hello world");
			op.completed_with_result ("result");

			assert_true (callback1_called);
			assert_true (callback2_called);
		}

		private void test_error_callback () {
			op.add_callback (callback3, "hello world");
			op.completed_with_error (new TestAsyncOperationError.RANDOM_ERROR ("oops"));

			assert_true (callback3_called);
		}


		public void callback1(PeopleUtils.AsyncOperation<string> op, void* user_data) {
			callback1_called = true;

			assert_true (user_data == null);
			assert_true (op.completed == true);
			assert_true (op.succeeded == true);
			assert_true (op.get_result() == "result");
		}

		public void callback2(PeopleUtils.AsyncOperation<string> op, void* user_data) {
			callback2_called = true;

			assert_true ((string) user_data == "hello world");
			assert_true (op.completed == true);
			assert_true (op.succeeded == true);
			assert_true (op.get_result() == "result");
		}

		public void callback3(PeopleUtils.AsyncOperation<string> op, void* user_data) {
			callback3_called = true;

			assert_true ((string) user_data == "hello world");
			assert_true (op.completed == true);
			assert_true (op.succeeded == false);

			try {
				op.get_result ();
				failure ();
			} catch (TestAsyncOperationError e) {
				success ();
				assert_true (e.message == "oops");
			}
		}

		static int main(string[] args) {
			var suite = new AsyncOperation();
			return suite.run();
		}

	}
}
