using GLib;
using PeopleUtils;


namespace PeopleTest {

	class AsyncSearch : TestIt.Suite {

		PeopleUtils.AsyncSearch<string> search;
		uint hits;
		bool done;

		protected override void initialize () {
			name = "PeopleUtils::AsyncSearch";
			add_test_case ("no early hit notification",
					test_no_callback_before_start);
			add_test_case ("hit notification",
					test_callback_after_start);
			add_test_case ("empty result",
					test_empty_result);
		}

		protected override void set_up () {
			search = new PeopleUtils.AsyncSearch<string> ();
			search.hit_found += hit_found_callback;
			search.done += done_callback;

			hits = 0;
			done = false;

			base.set_up ();
		}

		protected override void tear_down () {
			search = null;
			base.tear_down ();
		}


		private void test_no_callback_before_start () {
			search.add_hit ("Hello");
			assert_true (hits == 0);

			search.finish ();
			assert_true (hits == 0);
			assert_true (done == false);

			search.start ();
			assert_true (hits == 1);
			assert_true (done == true);
		}

		private void test_callback_after_start () {
			search.start();

			search.add_hit ("Hello");
			assert_true (hits == 1);
			assert_true (done == false);

			search.finish ();
			assert_true (hits == 1);
			assert_true (done == true);
		}

		private void test_empty_result () {
			search.start();
			search.finish ();

			assert_true (hits == 0);
			assert_true (done == true);
		}

		private void hit_found_callback(PeopleUtils.AsyncSearch<string> search, constpointer hit) {
			hits++;

			assert_true ((string) hit == "Hello");
		}

		private void done_callback(PeopleUtils.AsyncSearch<string> search) {
			done = true;
		}

		static int main(string[] args) {
			var suite = new AsyncSearch();
			return suite.run();
		}
	}
}
