using GLib;
using PeopleUtils;

namespace PeopleTest {

	errordomain PeopleTestError {
		RANDOM_ERROR = 1,
	}

	class MultiAsyncOperation : TestIt.Suite {

		AsyncOperation<string> operation1;
		AsyncOperation<string> operation2;
		AsyncOperation<string> operation3;
		PeopleUtils.MultiAsyncOperation<string> multi_operation;
		bool callback_called;

		protected override void initialize () {
			name = "PeopleUtils::MultiAsyncOperation";
			add_test_case ("early callback", test_early_callback);
			add_test_case ("late callback", test_late_callback);
			add_test_case ("middle callback", test_middle_callback);
		}

		protected override void set_up () {
			var operations = new Gee.ArrayList<AsyncOperation<string> > ();
			operation1 = new AsyncOperation<string>();
			operation2 = new AsyncOperation<string>();
			operation3 = new AsyncOperation<string>();

			operations.add (operation1);
			operations.add (operation2);
			operations.add (operation3);

			multi_operation = new PeopleUtils.MultiAsyncOperation<string> (operations);
			callback_called = false;
			base.set_up ();
		}

		protected override void tear_down () {
			operation1 = null;
			operation2 = null;
			operation3 = null;

			multi_operation = null;
			base.tear_down ();
		}

		private void test_early_callback () {
			multi_operation.add_callback (test_simple_case_callback);

			operation1.completed_with_result ("Hello");
			operation2.completed_with_error (new PeopleTestError.RANDOM_ERROR("oops"));
			operation3.completed_with_result ("world");

			assert_true (callback_called);
		}

		private void test_late_callback () {
			operation1.completed_with_result ("Hello");
			operation2.completed_with_error (new PeopleTestError.RANDOM_ERROR("oops"));
			operation3.completed_with_result ("world");

			multi_operation.add_callback (test_simple_case_callback);

			assert_true (callback_called);
		}

		private void test_middle_callback () {
			operation1.completed_with_result ("Hello");
			operation2.completed_with_error (new PeopleTestError.RANDOM_ERROR("oops"));

			multi_operation.add_callback (test_simple_case_callback);

			operation3.completed_with_result ("world");

			assert_true (callback_called);
		}

		private void test_simple_case_callback(PeopleUtils.MultiAsyncOperation<string> op) {
			callback_called = true;

			try {
				var result = op.get_result ();
				assert_true (result.get (0) == "Hello");
				assert_true (result.get (1) == null);
				assert_true (result.get (2) == "world");
			} catch (Error e) {
				failure ();
			}
		}


		static int main(string[] args) {
			var suite = new MultiAsyncOperation();
			return suite.run();
		}
	}
}
