using GLib;
using PeopleUtils;


namespace PeopleTest {

	class MultiAsyncSearch : TestIt.Suite {

		AsyncSearch<string> search1;
		AsyncSearch<string> search2;
		PeopleUtils.MultiAsyncSearch<string, string> multi_search;

		Gee.Set<string> hits;
		uint count_hit_found_cb_called;
		uint count_done_cb_called;

		protected override void initialize () {
			name = "PeopleUtils::MultiAsyncSearch";
			add_test_case ("early start", test_early_start);
			add_test_case ("late start", test_late_start);
			add_test_case ("middle start", test_middle_start);
		}

		protected override void set_up () {
			var searches = new Gee.HashMap<AsyncSearch, string> ();
			search1 = new AsyncSearch<string> ();
			search2 = new AsyncSearch<string> ();

			searches.set (search1, "");
			searches.set (search2, "");

			multi_search = new PeopleUtils.MultiAsyncSearch<string, string> (searches);

			hits = new Gee.HashSet<string> (str_hash, str_equal);
			count_hit_found_cb_called = 0;
			count_done_cb_called = 0;

 			multi_search.hit_found += hit_found_callback;
			multi_search.done += done_callback;

			base.set_up ();
		}

		protected override void tear_down () {
			search1 = null;
			search2 = null;

			multi_search = null;
			base.tear_down ();
		}

		private void test_early_start () {
			multi_search.start ();
			search1.add_hit ("Hello");
			search1.add_hit ("crazy");
			search1.finish ();
			search2.add_hit ("world");
			search2.finish ();
			multi_search.stop ();

			assert_true (count_hit_found_cb_called == 3);
			assert_true (count_done_cb_called == 1);
		}

		private void test_late_start () {
			search1.add_hit ("Hello");
			search1.add_hit ("crazy");
			search1.finish ();
			search2.add_hit ("world");
			search2.finish ();

			assert_true (count_hit_found_cb_called == 0);

			multi_search.start ();
			multi_search.stop ();

			assert_true (count_hit_found_cb_called == 3);
			assert_true (count_done_cb_called == 1);
		}

		private void test_middle_start () {
			search1.add_hit ("Hello");

			assert_true (count_hit_found_cb_called == 0);

			multi_search.start ();

			assert_true (count_hit_found_cb_called == 1);

			search1.add_hit ("crazy");
			search1.finish ();
			search2.add_hit ("world");
			search2.finish ();

			multi_search.stop();

			assert_true (count_hit_found_cb_called == 3);
			assert_true (count_done_cb_called == 1);
		}

		private void hit_found_callback(PeopleUtils.MultiAsyncSearch search, string source, string hit) {
			count_hit_found_cb_called++;
			hits.add (hit);
		}

		private void done_callback(PeopleUtils.MultiAsyncSearch search) {
			count_done_cb_called++;

			assert_true (hits.contains ("Hello"));
			assert_true (hits.contains ("crazy"));
			assert_true (hits.contains ("world"));
		}

		static int main(string[] args) {
			var suite = new MultiAsyncSearch();
			return suite.run();
		}
	}
}
