/* abstract-contact-source.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;
using Gee;
using PeopleBackend;


namespace PeopleBackendHelper {
	/**
	 * AbstractContactSource is an abstract class implementing
	 * {@link ContactSource}.
	 * <p>
	 * This stub of implementation provides a helper method to deal with
	 * the signal of this ContactSource.
	 * <p>
	 * This stub of implementation extends {@link AsyncOperationHelper} to
	 * provide methods to deal with {@link AsyncOperation} creation and
	 * completion.
	 */
	public abstract class AbstractContactSource : AsyncOperationHelper,
			ContactSource {
		/* ContactSource interface */
		public abstract ContactSourceFlags flags {
			get;
			protected set;
			default = ContactSourceFlags.NONE;
		}

		public abstract AsyncOperation<Contact> request_contact(string? id);
		/* end of ContactSource interface */

		/**
		 * Binds the given {@link Contact} to the signals of this
		 * {@link ContactSource}.
		 *
		 * @param contact the Contact to bind
		 * @see           ContactSource#contact_added
		 * @see           ContactSource#contact_removed
		 */
		protected void bind_contact(Contact contact) {
			contact.notify += contact_state_changed;
		}

		private void contact_state_changed(Contact contact, ParamSpec pspec) {
			if(pspec.name == "id") {
				if(contact.id == null)
					contact_removed(contact);
				else
					contact_added(contact);
			}
		}
	}
}
