/* abstract-contact.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;
using Gee;
using PeopleBackend;


namespace PeopleBackendHelper {
	/**
	 * AbstractContact is an abstract class implementing {@link Contact}.
	 * <p>
	 * This stub of implementation extends {@link AsyncOperationHelper} to
	 * provide methods to deal with {@link AsyncOperation} creation and
	 * completion.
	 */
	public abstract class AbstractContact : AsyncOperationHelper, Contact {
		[Notify]
		public ContactFlags flags {
			get;
			protected set construct;
			default = ContactFlags.NONE;
		}

		[Notify]
		public string?# id {
			get;
			protected set construct;
			default = null;
		}

		/* Contact interface methods */
		public abstract AsyncOperation<Gee.Set<string> > get_defined_fields();

		public abstract AsyncOperation<Gee.Set<string> > get_definable_fields();

		public abstract AsyncOperation<Gee.Set<string> > get_mandatory_fields();

		public abstract AsyncOperation<ContactFieldSpec> get_field_spec(string field_name);

		public abstract AsyncOperation<Gee.Map<string, string> >
				get_fields(Gee.Set<string> fields);

		public abstract AsyncOperation<Gee.Set<string> >
				set_fields(Gee.Map<string, string> fields);

		public abstract AsyncOperation<bool> @delete();
		/* end of Contact interface methods */
	}
}
