using GLib;
using Gee;
using PeopleBackend;
using PeopleBackendHelper;


class AbstractContactSourceStub : AbstractContactSource {
	public override AsyncOperation<Contact> request_contact(string? id=null) {
		return new AsyncOperation<Contact> ();
	}

	public void do_bind_contact(Contact contact) {
		bind_contact (contact);
	}
}

class AbstractContactSourceFactoryStub : AbstractContactSourceFactory {
	public override ContactSourceParamSpec[] get_parameters () {
		var parameters = new ContactSourceParamSpec[2];
		parameters[0] = new ContactSourceParamSpec ("username",
				typeof(string), ContactSourceParamFlags.MANDATORY);
		parameters[1] = new ContactSourceParamSpec ("password",
				typeof(string), ContactSourceParamFlags.NONE);
		return parameters;
	}

	public void do_call_check_parameters(Map<string, string>? parameters)
			throws ContactSourceFactoryError {
		check_parameters(parameters);
	}

	public override ContactSource request_contact_source (Map<string, string>? parameters)
			throws ContactSourceFactoryError {
		return new AbstractContactSourceStub ();
	}
}

private class TestAbstractContactSourceFactory {

	public static int main (string[] args) {
		var tester = new SelfTest ("PeopleBackendHelper::AbstractContactSourceFactory testing");
		tester.start ();

		var factory = new AbstractContactSourceFactoryStub();

		try {
			var args = new HashMap<string, string> (str_hash, str_equal, str_equal);

			args.set ("username", "kimbix");
			args.set ("password", "linox45");

			factory.do_call_check_parameters(args);
			tester.success ("checking check_parameters(ok) exceptions");
		} catch (PeopleBackend.ContactSourceFactoryError e) {
			tester.failure ("checking check_parameters(ok) exceptions");
		}

		try {
			var args = new HashMap<string, string> (str_hash, str_equal, str_equal);

			args.set ("password", "linox45");

			factory.do_call_check_parameters(args);
			tester.failure ("checking check_parameters(missing) exceptions");
		} catch (PeopleBackend.ContactSourceFactoryError e) {
			tester.success ("checking check_parameters(missing) exceptions");
		}

		try {
			factory.do_call_check_parameters(null);
			tester.failure ("checking check_parameters(null) exceptions");
		} catch (PeopleBackend.ContactSourceFactoryError e) {
			tester.success ("checking check_parameters(null) exceptions");
		}

		if (tester.end ())
			return 0;
		return 1;
	}

}
