using GLib;
using Gee;
using PeopleBackend;
using PeopleBackendHelper;

private class AsyncOperationHelperExtension : AsyncOperationHelper {

	public Set<AsyncOperation> do_get_pending_operations() {
		return pending_operations;
	}

	public AsyncOperation<Collection> do_sync_result(Object object) {
		return sync_result(object);
	}

	public AsyncOperation<bool> do_sync_result_bool(bool value) {
		return sync_result_bool(value);
	}

	public AsyncOperation<void*> do_sync_error(Error error) {
		return sync_error(error);
	}


	public AsyncOperation<Object> do_async_operation_init() {
		return async_operation_init();
	}

	public void do_async_operation_emit(AsyncOperation<Object> op,
			Object object) {
		async_operation_emit(op, object);
	}


	public AsyncOperation<bool> do_async_operation_init_bool() {
		return async_operation_init_bool();
	}

	public void do_async_operation_emit_bool(AsyncOperation<bool> op, bool value) {
		async_operation_emit_bool(op, value);
	}

	public void do_async_operation_error(AsyncOperation<void*> op,
			Error error) {
		async_operation_error(op, error);
	}

}

public errordomain TestAbstractContactError {
	RANDOM_ERROR = 1,
}

private class TestAbstractContact {

	public static int main (string[] args) {
		var tester = new SelfTest ("PeopleBackendHelper::AsyncOperationHelper testing");
		tester.start ();

		AsyncOperationHelperExtension helper = new AsyncOperationHelperExtension();

		/* synchronous collection result */
		Collection collection1 = new ArrayList<void*>();
		tester.assert_true("checking synchronous collection result",
			helper.do_sync_result(collection1).get_result() == collection1);

		/* synchronous bool result */
		tester.assert_true("checking synchronous true result",
			helper.do_sync_result_bool(true).get_result() == true);
		tester.assert_true("checking synchronous false result",
			helper.do_sync_result_bool(false).get_result() == false);

		/* synchronous error */
		try {
			helper.do_sync_error(new TestAbstractContactError.RANDOM_ERROR("oops")).get_result();
			tester.failure("checking synchronous error throw");
			tester.failure("checking synchronous error message");
		} catch (TestAbstractContactError e) {
			tester.success("checking synchronous error throw");
			tester.assert_true("checking synchronous error message",
					e.message == "oops");
		}

		/* asynchronous collection result */
		tester.assert_true("checking before size of pending async operations set",
			helper.do_get_pending_operations().size == 0);

		var op1 = helper.do_async_operation_init();
		tester.assert_true("checking during size of pending async operations set",
			helper.do_get_pending_operations().size == 1);

		Collection collection2 = new ArrayList<void*>();
		helper.do_async_operation_emit(op1, collection2);
		tester.assert_true("checking asynchronous collection result",
			(Collection) op1.get_result() == collection2);
		tester.assert_true("checking after size of pending async operations set",
			helper.do_get_pending_operations().size == 0);

		/* asynchronous bool result */
		tester.assert_true("checking before size of pending async operations set",
			helper.do_get_pending_operations().size == 0);

		var op2 = helper.do_async_operation_init_bool();
		tester.assert_true("checking during size of pending async operations set",
			helper.do_get_pending_operations().size == 1);

		helper.do_async_operation_emit_bool(op2, true);
		tester.assert_true("checking asynchronous true result",
			op2.get_result() == true);
		tester.assert_true("checking after size of pending async operations set",
			helper.do_get_pending_operations().size == 0);


		tester.assert_true("checking before size of pending async operations set",
			helper.do_get_pending_operations().size == 0);

		var op3 = helper.do_async_operation_init_bool();
		tester.assert_true("checking during size of pending async operations set",
			helper.do_get_pending_operations().size == 1);

		helper.do_async_operation_emit_bool(op3, false);
		tester.assert_true("checking asynchronous false result",
			op3.get_result() == false);
		tester.assert_true("checking after size of pending async operations set",
			helper.do_get_pending_operations().size == 0);

		/* asynchronous error */
		tester.assert_true("checking before size of pending async operations set",
			helper.do_get_pending_operations().size == 0);

		var op4 = helper.do_async_operation_init_bool();
		tester.assert_true("checking during size of pending async operations set",
			helper.do_get_pending_operations().size == 1);

		helper.do_async_operation_error(op4, new TestAbstractContactError.RANDOM_ERROR("oops"));
		try {
			op4.get_result();
			tester.failure("checking asynchronous error throw");
			tester.failure("checking asynchronous error message");
		} catch (TestAbstractContactError e) {
			tester.success("checking asynchronous error throw");
			tester.assert_true("checking asynchronous error message",
					e.message == "oops");
		}

		tester.assert_true("checking after size of pending async operations set",
			helper.do_get_pending_operations().size == 0);

		if (tester.end ())
			return 0;
		return 1;
	}

}
