/* query.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;
using Gee;

namespace PeopleBackend {
	public enum Operator {
		NOT,
		AND,
		OR,
		EQUALS,
		CONTAINS,
	}

	public interface QueryVisitor : Object {
		public abstract void visit_composite (CompositeQuery query);
		public abstract void visit_field (FieldQuery query);
		public abstract void visit_value (ValueQuery query);
	}

	public abstract class Query : Object {
		public abstract void accept (QueryVisitor visitor);

		public static Query not (Query query) {
			var queries = new ArrayList<Query> ();
			queries.add (query);
			return new CompositeQuery (Operator.NOT, queries);
		}

		public static Query and (Gee.List<Query> queries) {
			return compose (Operator.AND, queries);
		}

		public static Query or (Gee.List<Query> queries) {
			return compose (Operator.OR, queries);
		}

		private static Query compose (Operator op, Gee.List<Query> queries) {
			return new CompositeQuery (op, queries);
		}
	}

	public class CompositeQuery : Query {
		public Operator operator {
			public get;
			private construct set;
		}

		public Gee.List<Query> queries {
			public get;
			private construct set;
		}

		public CompositeQuery (Operator operator, Gee.List<Query> queries) {
			this.operator = operator;
			this.queries = queries;
		}

		public override void accept (QueryVisitor visitor) {
			visitor.visit_composite (this);
		}
	}

	public class FieldQuery : Query {
		public string field {
			public get;
			private construct set;
		}

		public FieldQuery (string field) {
			this.field = field;
		}

		public override void accept (QueryVisitor visitor) {
			visitor.visit_field (this);
		}
	}

	// TODO: use a Value instead of a string
	public class ValueQuery : Query {
		public string value {
			public get;
			private construct set;
		}

		public ValueQuery (string value) {
			this.value = value;
		}

		public override void accept (QueryVisitor visitor) {
			visitor.visit_value (this);
		}
	}
}
