/* address-book-manager.vala
 *
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;
using Gee;


namespace People {

	public class AddressBookManager : Object {

		public Config config {
			get;
			set construct;
		}

		private Map<string, PeopleBackend.ContactSourceFactory> backends;
		private Map<string, AddressBook> address_books;

		construct {
			backends = new HashMap<string, PeopleBackend.ContactSourceFactory>(
					str_hash, str_equal);
			backends.set("dummy", new PeopleBackendDummy.ContactSourceFactory());
			backends.set("keyfile", new PeopleBackendKeyfile.ContactSourceFactory());
			backends.set("sqlite", new PeopleBackendSqlite.ContactSourceFactory());

			address_books = new HashMap<string, AddressBook>(str_hash, str_equal);

			foreach (weak string address_book_id in config.list_address_books()) {
				var source_ids = config.get_address_book_sources(address_book_id);
				Map<string, PeopleBackend.ContactSource> sources =
					new HashMap<string, PeopleBackend.ContactSource>(str_hash, str_equal);

				foreach (weak string source_id in source_ids) {
					var source_class = config.get_contact_source_class(source_id);
					var source_parameters = config.get_contact_source_parameters(source_id);
					var factory = backends.get(source_class);
					try {
						var source = factory.request_contact_source(source_parameters);
						sources.set(source_id, source);
					} catch (PeopleBackend.ContactSourceFactoryError e) {
						warning("Failed to request contact source '%s' (%s)",
							source_id, e.message);
					}
				}

				var address_book = new AddressBook(
						new BiMap<string, PeopleBackend.ContactSource>(sources),
						new HashSet<Gee.List<string> > ());
				address_books.set(address_book_id, address_book);
			}
		}

		public AddressBookManager(Config config) {
			this.config = config;
		}

		public Set<string> list_address_books() {
			return address_books.get_keys();
		}

		public AddressBook get_address_book(string address_book_id) {
			return address_books.get(address_book_id);
		}
	}
}
