/* address-book.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 * Copyright (C) 2008 Ali Sabil <ali.sabil@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;
using Gee;
using PeopleBackend;

namespace People {

	public class AddressBook : Object {
		public People.BiMap<string, PeopleBackend.ContactSource> sources {
			public get;
			private construct set;
		}

		public Gee.Set<Gee.List<string> > merged_contacts {
			public get;
			private construct set;
		}

		public AddressBook(People.BiMap<string, PeopleBackend.ContactSource> sources,
				Gee.Set<Gee.List<string> > merged_contacts) {
			this.sources = sources;
			this.merged_contacts = merged_contacts;
		}

		/* Public API */
		public QueryResult query () {
			var searches = new Gee.HashMap<AsyncSearch, ContactSource> ();
			foreach (PeopleBackend.ContactSource source in sources.get_y_set ()) {
				var search = ((PeopleBackend.DumpableContactSource) source).dump ();
				searches.set (search, source);
			}
			var search = new MultiAsyncSearch<PeopleBackend.ContactSource,
					PeopleBackend.Contact> (searches);

			return new QueryResult (this, search);
		}
		/* End of public API */
	}

}
