/* contact.vala
 *
 * Copyright (C) 2008 Johann Prieur <johann.prieur@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 */

using GLib;
using Gee;
using PeopleBackend;
using PeopleBackendHelper;

namespace People {

	public class Contact : PeopleBackendHelper.AsyncOperationHelper {
		private Set<MultiAsyncOperation> pending_multi_operations =
				new Gee.HashSet<MultiAsyncOperation> ();

		public string id {
			public get;
			private construct set;
		}

		public Gee.List<PeopleBackend.Contact> contacts {
			private get;
			private set;
		}

		public Contact (string id, Gee.List<PeopleBackend.Contact> contacts) {
			this.id = id;
			this.contacts = contacts;
		}

		/* Public API */
		public AsyncOperation<Gee.Set<string> > get_defined_fields () {
			var op = async_operation_init ();

			var ops = new Gee.ArrayList<AsyncOperation<Gee.Set <string> > > ();
			foreach (PeopleBackend.Contact contact in contacts) {
				ops.add (contact.get_defined_fields ());
			}

			var multi = new MultiAsyncOperation<Gee.Set<string> > (ops);
			multi.add_callback (get_defined_fields_callback, op);
			pending_multi_operations.add (multi);

			return op;
		}

		private void get_defined_fields_callback(People.MultiAsyncOperation<Gee.Set<string> > multi,
				void* data) {
			var results = new Gee.HashSet<string> (str_hash, str_equal);
			AsyncOperation op = (AsyncOperation) data;

			foreach (Set<string> result in multi.get_result ()) {
				foreach (string r in result) {
					results.add (r);
				}
			}

			async_operation_emit (op, results);
			pending_multi_operations.remove (multi);
		}

		public AsyncOperation<Gee.Map<string, string> > get_fields (Gee.Set<string> fields) {
			var op = async_operation_init ();

			var ops = new Gee.ArrayList<AsyncOperation<Gee.Map <string, string> > > ();
			foreach (PeopleBackend.Contact contact in contacts) {
				ops.add (contact.get_fields (fields));
			}

			var multi = new MultiAsyncOperation<Gee.Map<string, string> > (ops);
			multi.add_callback (get_fields_callback, op);
			pending_multi_operations.add (multi);

			return op;
		}

		private void get_fields_callback(People.MultiAsyncOperation<Gee.Map<string, string> > multi,
				void* data) {
			var results = new Gee.HashMap<string, string> (str_hash, str_equal, str_equal);
			AsyncOperation op = (AsyncOperation) data;

			foreach (Map<string, string> result in multi.get_result ()) {
				foreach (string key in result.get_keys ()) {
					results.set (key, result.get (key));
				}
			}

			async_operation_emit (op, results);
			pending_multi_operations.remove (multi);
		}
		/* End of public API */
	}

}
