using GLib;
using Gee;
using People;

namespace PeopleTest {

	class BiMap : TestIt.Suite {
		People.BiMap<string, string> filled_bimap;
		People.BiMap<string, string> empty_bimap;

		protected override void initialize () {
			name = "People::BiMap";
			add_test_case ("test get (filled)", test_get_filled);
			add_test_case ("test get (empty)", test_get_empty);
			add_test_case ("test set x (new)", test_set_x_new);
			add_test_case ("test set x (overwrite)", test_set_x_overwrite);
			add_test_case ("test set y (new)", test_set_y_new);
			add_test_case ("test set y (overwrite)", test_set_y_overwrite);
		}

		protected override void set_up () {
			var map = new Gee.HashMap<string, string> (str_hash, str_equal, str_equal);
			map.set ("bu", "bo");
			map.set ("bo", "ba");

			filled_bimap = new People.BiMap<string, string> (map,
					str_hash, str_equal, str_hash, str_equal);

			empty_bimap = new People.BiMap<string, string> (null,
					str_hash, str_equal, str_hash, str_equal);
			base.set_up ();
		}

		protected override void tear_down () {
			filled_bimap = null;

			base.tear_down ();
		}

		private void test_get_filled () {
			assert_true (filled_bimap.get_x_set ().size == 2);
			assert_true (filled_bimap.get_y_set ().size == 2);

			assert_true (filled_bimap.get_y ("bu") == "bo");
			assert_true (filled_bimap.get_y ("bo") == "ba");
			assert_true (filled_bimap.get_x ("bo") == "bu");
			assert_true (filled_bimap.get_x ("ba") == "bo");
		}

		private void test_get_empty () {
			assert_true (empty_bimap.get_x_set ().size == 0);
			assert_true (empty_bimap.get_y_set ().size == 0);

			assert_true (empty_bimap.get_y ("bu") == null);
			assert_true (empty_bimap.get_x ("ba") == null);
		}

		private void test_set_x_new () {
			/* set new association from x */
			filled_bimap.set_x ("bon", "bam");
			assert_true (filled_bimap.get_y ("bon") == "bam");
			assert_true (filled_bimap.get_x ("bam") == "bon");
			assert_true (filled_bimap.get_x_set ().size == 3);
			assert_true (filled_bimap.get_y_set ().size == 3);
		}

		private void test_set_x_overwrite () {
			/* change existing association from x */
			filled_bimap.set_x ("bo", "bim");
			assert_true (filled_bimap.get_y ("bo") == "bim");
			assert_true (filled_bimap.get_x ("bim") == "bo");
			assert_true (filled_bimap.get_x_set ().size == 2);
			assert_true (filled_bimap.get_y_set ().size == 2);
		}

		private void test_set_y_new () {
			/* set new association from y */
			filled_bimap.set_y ("bon", "bam");
			assert_true (filled_bimap.get_x ("bon") == "bam");
			assert_true (filled_bimap.get_y ("bam") == "bon");
			assert_true (filled_bimap.get_y_set ().size == 3);
			assert_true (filled_bimap.get_x_set ().size == 3);
		}

		private void test_set_y_overwrite () {
			/* change existing association from y */
			filled_bimap.set_y ("bo", "bim");
			assert_true (filled_bimap.get_x ("bo") == "bim");
			assert_true (filled_bimap.get_y ("bim") == "bo");
			assert_true (filled_bimap.get_y_set ().size == 2);
			assert_true (filled_bimap.get_x_set ().size == 2);
		}

		static int main (string[] args) {
			var suite = new BiMap ();
			return suite.run();
		}
	}
}
